FileAttributes = [      # file attributes which can be used in queries
            "creator", "created_timestamp", "name", "namespace", "size",
            "retired", "retired_by", "retired_timestamp", "updated_by",  
            "updated_timestamp"
]  

DatasetAttributes = [
    "creator", "created_timestamp", "namespace", "name",
    "description", "frozen", "monotonic"
]
